;	include		"calc.h"
;
;	include		"doorsos.h"
;	include		"hexlib.h"
;	include		"macros.h"
;
;	include		"prosit.h"
;	include		"winlib.h"
;
;	include		"keycodes.h"
;
;	xdef		_library
;
;	xdef		guilib@0000	;Text_Button


;****************************************************
;* Input: d5.w0 = Object flags                      *
;*        d5.w1 = System flags                      *
;*        d6.w0 = Object Y1                         *
;*        d6.w1 = Object X1                         *
;*        d7.w0 = Object Y2                         *
;*        d7.w1 = Object X2                         *
;*        a3.l = *Window                            *
;*        a4.l = *Object                            *
;*        a6.l = *Window owner's datareturn area    *
;*                                                  *
;* Output: a3.l = *Window                           *
;*         a4.l = *Object                           *
;*         a5.l = *Next object (used by Initialize, *
;*          ignored by the rest of the GUI system)  *
;*         d4.l = Repeat delay (if used)            *
;*         d5 = Flags (if changed)                  *
;*         d6.w0 = Object Y1                        *
;*         d6.w1 = Object X1                        *
;*         d7.w0 = Object Y2                        *
;*         d7.w1 = Object X2                        *
;****************************************************


;High byte (GUI system flags from return value, can of course
; also be set on the first call to the object):
;1xxxxxxx = Trigger event on first click
;x1xxxxxx = Use repeat calls
;xx1xxxxx = Trigger event on repeat calls
;xxx1xxxx = Trigger event on release
;xxxx0xxx = Event = "moub"
;xxxx1xxx = Event = "quit"

;Lower byte (object flags)
;1xxxxxxx = Initialize
;0xxxxxxx = Action (by GUI-handler)
;x1xx11xx = First click
;xx1xxxxx = Mousebutton released
;xxx1xxxx = Mousepointer moved
;xxxx1xxx = Mousepointer moved inside <-> outside
;xxxxx0xx = "Outside"
;xxxxx1xx = "Inside"
;xxxxxx1x = Repeat call
;xxxxxxx1 = Force triggering of event on return

;**********
;* BUTTON *
;**********
guilib__Button:
;guilib@0000:
	move.l		d5,d3
	swap		d3
	and.b		#%11101000,d3	;Bits that cause button drawing
	beq		\nodraw		;No flag that requires redrawing is set, skip that part

	move.l		a4,a5		;*object (after flags)

	btst.l		#23,d5		;Initializing?
	beq		\gfx		;No, filling or contents isn't used!

;* DRAW FILLING, FRAME AND CONTENTS *

;* BUTTON FILLING *
	move.l		d7,-(sp)		;Y2/X2
	move.l		d6,-(sp)		;Y1/X1
	move.w		#A_REVERSE,-(sp)	;Flags
	pea		2(sp)			;*WIN_RECT
	move.l		a3,-(sp)		;Window
	btst.l		#9,d5		;Fill bit set?
	beq.s		\oframe			;No, go to outer frame
	btst.l		#8,d5		;Fill bit set - color?
	bne.s		\filld			;Color = white
	move.w		#A_NORMAL,8(sp)		;Color = black
\filld	jsr		doorsos::WinFill

;* OUTER BUTTON FRAME *
\oframe	move.w		#A_REVERSE,8(sp)		;Flags
	btst.l		#7,d5		;Frame bit set?
	beq.s		\noofr			;No, go to button contents
	btst.l		#6,d5		;Frame bit set - color?
	bne.s		\ofrmed			;Color = white
	move.w		#A_NORMAL,8(sp)	;Color = black
\ofrmed	jsr		doorsos::WinRect
\noofr	lea		18(sp),sp

;* BUTTON CONTENTS *
	btst.l		#11,d5
	beq.s		\gfx		;Inside may be GFX
;* BUTTON CONTENTS = TEXT *
	move.w		d5,d3
	and.w		#%0000000000011000,d3
	lsr.w		#3,d3			;Font
	move.w		d3,-(sp)
	move.l		a3,-(sp)
	jsr		doorsos::WinFont
	tst.b		(a5)
	bne.s		\inline		;Text follows directly
	move.l		(a5)+,(sp)	;Text is pointed to, get *string
	bra.s		\len
\inline	move.l		a5,(sp)		;String is inline, use pointer directly
\loop	tst.b		(a5)+
	bne.s		\loop		;Find end of string
	move.w		a5,d1
	btst.b		#0,d1
	beq.s		\len		;Pointer is aligned
	addq.l		#1,a5		;Pointer isn't aligned, align it
;Find string dimensions
\len	jsr		doorsos::DrawStrWidth
	addq.w		#1,d0		;Double the left blank pixel column in text
	swap		d0		;d0.w1 = String width + 1
	move.w		d3,d0
	lsl.w		#1,d0
	addq.w		#5,d0		;d0.w0 = String height

	move.b		d5,d3
	and.b		#%00000111,d3
	move.b		d3,W_TEXT_LINE_ATTRIBUTES(a3)
	bra.s		\plot

;* BUTTON CONTENTS = GFX *
\gfx	btst.l		#10,d5
	beq		\iframe		;Inside is not gfx, it is empty!

	move.w		d5,d0
	and.w		#%0000000000000111,d0
	move.w		d0,-(sp)	;Flags

	move.l		(a4),a0		;*gfx
	and.b		#%10100000,d3	;On initialize and mousebutton released
	bne		\usegfx		; we always draw a raised button.
	btst.l		#18,d5		;If none of those are set it's the "inside/outside"
	beq.s		\usegfx		; bit that determines it.
	move.l		4(a4),a0	;*pressed
\usegfx	move.l		a0,-(sp)	;*gfx
	lea		8(a4),a5	;*Next object
	move.l		(a0),d0		;Width / Heigth

;* DEAL WITH CENTERING AND JUSTIFYING *
\plot	move.l		d7,d3
	sub.l		d6,d3
	add.l		#$00010001,d3	;Width/Height
	move.l		d6,-(sp)	;X/Y

	btst.l		#7,d5		;Frame bit set?
	beq.s		\ifr		;No, no adjustment
	add.l		#$00010001,(sp)	;Increase X/Y
	sub.l		#$00020002,d3	;Decrease Width/Height
\ifr	btst.l		#5,d5		;3D bit set?
	beq.s		\ifrd		;No, no adjustment
	add.l		#$00010001,(sp)	;Increase X/Y
	sub.l		#$00020002,d3	;Decrease Width/Height
\ifrd	sub.l		d0,d3		;Subtract object contents size from frame size
	swap		d3		;Get Width into lower word
	btst.l		#13,d5		;Horizontally centered?
	beq.s		\hjust		;Nope
	asr.w		#1,d3
	bra.s		\vert
\hjust	btst.l		#12,d5		;Left- or right-justified?
	bne.s		\vert		;Right, we're done
	move.w		#0,d3

\vert	swap		d3		;Get Height into lower word
	btst.l		#15,d5		;Vertically centered?
	beq.s		\vjust		;Nope
	asr.w		#1,d3
	bra.s		\hvdone
\vjust	btst.l		#14,d5		;Top- or bottom-justified?
	bne.s		\hvdone		;Right, we're done
	move.w		#0,d3

\hvdone	add.l		d3,(sp)		;Adjust X/Y

	move.l		a3,-(sp)	;*Window
	btst.l		#11,d5
	beq.s		\plgfx		;Inside is GFX

	jsr		doorsos::WinStrXY
	lea		12+2(sp),sp
	bra.s		\iframe

\plgfx	jsr		doorsos::WinBitmapPut
	lea		14(sp),sp


;* INNER BUTTON FRAME *
\iframe	btst.l		#5,d5		;3D bit set?
	beq.s		\donedr		;No, no inner frame

;RAISED FRAME
	move.l		#A_REVERSE*65536+A_NORMAL,d3	;Assume "outside"
	move.l		d5,d0
	swap		d0
	and.b		#%10100000,d0	;On initialize and mousebutton released
	bne.s		\ifplot		; we always draw a raised button.
	btst.l		#18,d5		;If none of those are set it's the "inside/outside"
	beq.s		\ifplot		; bit that determines it.
	move.l		#A_NORMAL*65536+A_REVERSE,d3
\ifplot	move.l		d7,-(sp)		;Y2/X2
	sub.l		#$00010001,(sp)
	move.l		d6,-(sp)		;Y1/X1
	add.l		#$00010001,(sp)
	move.w		d3,-(sp)		;Flags
	pea		2(sp)			;*WIN_RECT
	move.l		a3,-(sp)		;Window
	jsr		doorsos::WinRect
	swap		d3
	move.b		d3,W_TEXT_LINE_ATTRIBUTES(a3)
	move.w		10(sp),d3		;X1
	swap		d3
	move.w		10+6(sp),d3		;Y2
	move.w		10+2(sp),10+6(sp)	;Set Y2 = Y1
	jsr		doorsos::WinLine
	move.l		d3,10+4(sp)
	jsr		doorsos::WinLine
	lea		18(sp),sp



;DONE DRAWING, REFRESH WINDOW?
\donedr	btst.l		#23,d5
	bne.s		\isinit		;Initialize, don't refresh!
	move.l		a3,-(sp)	;Window
	bsr		Refresh_Window
	addq.l		#4,sp
\isinit

;Now for the management part :o)
\nodraw	btst.l		#17,d5	;Is this a repeat call?
	beq.s		\done	;No, don't set / alter delay value!
	move.l		#100,d4	;Primary delay value
	btst.l		#22,d5	;Is this the first call?
	bne.s		\done	;Yes, use primary delay value
	move.l		#30,d4	;No, use secondary delay value

\done	rts






guilib__Horiz_Slider:
guilib__Vert_Slider:


;**************************************************
;* TEXT                                           *
;* Plot text at (x,y) = d2 using flags/font in d3 *
;**************************************************
guilib__Text:
	lea		5(a0),a1	;*Object + 5

	tst.b		(a1)
	bne.s		\inline		;String is included in the structure, go use it
;String is pointed to, get pointer
	addq.l		#1,a1
	move.w		a1,d0
	btst.b		#0,d0
	beq.s		\align		;Pointer is aligned
	addq.l		#1,a1		;Pointer isn't aligned, align it
\align	move.l		(a1)+,-(sp)
	bra.s		\plot
;String is included in object, use it
\inline	move.l		a1,-(sp)
\loop	tst.b		(a1)+
	bne.s		\loop

\plot	movem.l		d2-d3,-(sp)		;X/Y, Flags/Font
	move.w		d7,-(sp)		;Window

;	bsr		PlotText
	btst.b		#7,d1
	bne.s		\isinit		;Initialize, don't refresh!
	bsr		Refresh_Window
\isinit	lea		14(sp),sp

	rts

;********
;* LINE *
;********
guilib__Line:
	move.l		a4,a5		;*object (after flags)

	move.l		d7,-(sp)	;Y2/X2
	move.l		d6,-(sp)	;Y1/X1

	move.b		d5,W_TEXT_LINE_ATTRIBUTES(a3)	;Flags
	pea		0(sp)		;*WIN_RECT
	move.l		a3,-(sp)	;Window

	jsr		doorsos::WinLine

	btst.l		#23,d5
	bne.s		\isinit		;Initialize, don't refresh!
	bsr		Refresh_Window
\isinit	lea		16(sp),sp
	rts

;********
;* RECT *
;********
guilib__Rect:
	move.l		a4,a5		;*object (after flags)

	move.l		d7,-(sp)	;Y2/X2
	move.l		d6,-(sp)	;Y1/X1

	move.w		d5,-(sp)	;Flags
	pea		2(sp)		;*WIN_RECT
	move.l		a3,-(sp)	;Window

	jsr		doorsos::WinRect

	btst.l		#23,d5
	bne.s		\isinit		;Initialize, don't refresh!
	bsr		Refresh_Window
\isinit	lea		18(sp),sp
	rts

;********
;* RECT *
;********
guilib__Fill:
	move.l		a4,a5		;*object (after flags)

	move.l		d7,-(sp)	;Y2/X2
	move.l		d6,-(sp)	;Y1/X1

	move.w		d5,-(sp)	;Flags
	pea		2(sp)		;*WIN_RECT
	move.l		a3,-(sp)	;Window

	jsr		doorsos::WinFill

	btst.l		#23,d5
	bne.s		\isinit		;Initialize, don't refresh!
	bsr		Refresh_Window
\isinit	lea		18(sp),sp
	rts

;***********
;* SETCLIP *
;***********
guilib__SetClip:
	pea		-8(a0)		;*Object
	move.w		d7,-(sp)	;Window
	bsr		SetClip
	addq.l		#6,sp
	rts
