;*****************
;*               *
;* FILE ROUTINES *
;*               *
;*****************

;---------------------------------------------------------------------
; Get_File_Handle(BYTE *foldername, BYTE *filename)
;
; Function: Get the handle of a specific file
;
; Output: d5 = handle, 0 if file doesn't exist
;---------------------------------------------------------------------
Get_File_Handle:
prosit@0009:
	movem.l		a0-a4/d0-d1,-(sp)

	move.l		4+32(sp),a0
	move.l		4+28(sp),a1
	moveq		#0,d5		;In case file doesn't exist

;	TOGGLE_PROSIT

;Find folder
	move.l		doorsos::Heap,a2
	move.w		doorsos::FolderListHandle,d0
	lsl.w		#2,d0
	move.l		0(a2,d0.w),a2	;*Root
	addq.l		#2,a2
	move.w		(a2)+,d0	;Number of folders
	subq.w		#1,d0		;...-1
\loop1	move.l		a0,a3
	move.l		a2,a4
\cloop1	cmp.b		(a3)+,(a4)+
	bne.s		\next1
	tst.b		-1(a4)
	beq.s		\found1
	bra.s		\cloop1
\next1	lea		14(a2),a2	;Next entry
	dbra.s		d0,\loop1
	bra.s		\fail		;Folder not found

\found1	move.w		12(a2),d0
	doorsos::DEREF	d0,a2
	addq.l		#2,a2
	move.w		(a2)+,d0	;Number of files in folder
	subq.w		#1,d0		;...-1
\loop2	move.l		a1,a3
	move.l		a2,a4
\cloop2	cmp.b		(a3)+,(a4)+
	bne.s		\next2
	tst.b		-1(a4)
	beq.s		\found2
	bra.s		\cloop2
\next2	lea		14(a2),a2	;Next entry
	dbra.s		d0,\loop2
	bra.s		\fail		;File not found

\found2	move.w		12(a2),d5	;Found file, get handle

\fail	;TOGGLE_PROSIT

	movem.l		(sp)+,a0-a4/d0-d1
	rts

;---------------------------------------------------------------------
; Execute_File(WORD handle)
;
; Function: Execute a file.
; Output: d0.w = 0 if success, -1 if failed
;---------------------------------------------------------------------
Execute_File:
prosit@000A:
	movem.l		a0-a2/a5-a6/d0-d2/d4-d5,-(sp)

	move.w		4+10*4(sp),d5		;File handle

	TOGGLE_PROSIT

	bsr		RelocateFile	;a5=*_main, a6=*_file
	move.w		d4,d0
	blt.s		\fail

	jsr		(a5)		;Run file
	move.w		d5,d0
	move.l		($40),a0
	jsr		(a0)		;Unrelocate file

	tst.w		d4		;Copy of archived file?
	ble.s		\done		;No, we're done
	move.w		d5,-(sp)
	jsr		doorsos::HeapFree	;Yes, de-allocate it
	addq.l		#2,sp

\done	moveq		#0,d0		;Success

\fail	TOGGLE_PROSIT

	movem.l		(sp)+,a0-a2/a5-a6/d0-d2/d4-d5
	rts


;*********************************************************************
;* Relocate ASM file                                                 *
;* Input: d5.w = Handle of file                                      *
;* Output: a6.l = *File                                              *
;*         a5.l = *_main                                             *
;*         d4.w = 0 if success, -1 if failed, 1 if file was archived *
;*         d5.w = Handle of file, may be handle of copy of archive   *
;*********************************************************************
RelocateFile:
	movem.l		a0-a2/d0-d2,-(sp)

	move.w		d5,d0
	doorsos::DEREF	d0,a5		;Get adress of program variable

	moveq		#-1,d4		;Assume "failure"

	moveq		#0,d1
	move.w		(a5),d1
	lea		1(a5,d1.w),a2
	cmp.b		#$F3,(a2)	;is ASM ?
	bne		\Fail		;no -> bye !
	cmp.l		#"68kP",6(a5)	;is a DoorsOS/Plusshell program ?
	bne		\Fail		; no -> bye !
	tst.w		8+2(a5)		;Program already relocated?
	bgt		\Fail		; yes -> bye !

	moveq		#0,d4		;Return "success"

	cmp.l		#$200000,a5
	blt.s		\rel
;temporary unarchive an unarchived prog
	addq.l		#3,d1		:File size
	move.l		d1,-(sp)
	jsr		doorsos::HeapAllocHigh	;allocates a block to copy from Archive memory
	move.l		(sp)+,d1

	moveq		#-1,d4		;Assume "failure"
	move.w		d0,d5
	beq		\Fail		;Not enough memory

	move.l		a5,a0		;Store "real" adress of program
	doorsos::DEREF	d0,a6		;*Temporary program memory block
	move.l		a6,a5		;New adress of program
	subq.w		#1,d1
\cpy	move.b		(a0)+,(a6)+	;Copy program to temporary memory block
	dbra.s		d1,\cpy

	moveq		#1,d4		;Return "file was archived"

;Relocation
\rel	addq.l		#2,a5		;Program entry point
	move.l		a5,a6		;Store program adress

	move.w		$C(a5),d1	;Offset to _main
	lea		0(a5,d1.w),a5	;Program entry point (_main)

	move.w		d5,d0		;Program handle must be in d0 when relocating
	move.l		($38),a0	;Adress of relocation routine

	jsr		(a0)			;Relocate program

\Fail	movem.l		(sp)+,a0-a2/d0-d2
	rts
