;*****************
;*               *
;* LIST ROUTINES *
;*               *
;*****************

;---------------------------------------------------------------------
; List_Insert(BYTE *list, BYTE *structure, BYTE priority)
;
; Function: Insert <structure> in <list>. Its placement depends on
;  <priority>.
;---------------------------------------------------------------------
List_Insert:
prosit@000F:
	movem.l		a0-a3/d0,-(sp)

	move.l		4+5*4(sp),a1		;*list
	move.l		4+5*4+4(sp),a2		;*structure
;	move.w		4+5*4+8(sp),LIST_ID(a2)	;Set item ID
	move.b		4+5*4+8(sp),d0		;Priority

	move.b		d0,LIST_PRIORITY(a2)	;Set item priority
	tst.l		(a1)
	beq.s		\empty

;new item > first item?
	move.l		(a1),a0		;*First item
	cmp.b		LIST_PRIORITY(a0),d0
	blt.s		\find
	move.l		a2,(a1)		;Point list to new first item
	move.l		a0,-(sp)
	bsr		List_Previous
	addq.l		#4,sp
	bra.s		\insert

;Find first item that's <= new item...
\find	move.l		a0,a1			;Store *first
\loop	move.l		LIST_NEXT(a0),a3
	cmp.l		a3,a1
	beq.s		\insert			;End of list, insert item last
	cmp.b		LIST_PRIORITY(a3),d0
	bge.s		\insert			;Found item <= new item
	move.l		a3,a0
	bra.s		\loop

;If we don't find it we just insert it before the first item - if we don't alter the *first pointer
;then this is equal to inserting it last in the list!

;Insert new item after item (a0)
\insert	move.l		LIST_NEXT(a0),LIST_NEXT(a2)
	move.l		a2,LIST_NEXT(a0)	;Set *previous.*next = *structure
	bra.s		\done

\empty	move.l		a2,(a1)			;First item, set **list = *structure
	move.l		a2,LIST_NEXT(a2)	;...and *structure.*next = *structure
\done	movem.l		(sp)+,a0-a3/d0
	rts

;---------------------------------------------------------------------
; List_Remove(BYTE *list, BYTE *structure)
;
; Function: Remove <structure> from <list>.
;---------------------------------------------------------------------
List_Remove:
prosit@0010:
	movem.l		a0-a2,-(sp)
	move.l		4+3*4+4(sp),a2			;*structure
	move.l		4+3*4(sp),a1			;*list
	cmp.l		LIST_NEXT(a2),a2
	beq.s		\empty

	move.l		a2,-(sp)
	bsr		List_Previous			;a0 = *Previous
	addq.l		#4,sp
	move.l		LIST_NEXT(a2),LIST_NEXT(a0)	;Set *previous.*next = *structure.*next
	cmp.l		(a1),a2
	bne.s		\done				;Removed item wasn't first
	move.l		LIST_NEXT(a2),(a1)		;Removed item was first, update list header
	bra.s		\done
\empty	move.l		#0,(a1)
\done	movem.l		(sp)+,a0-a2
	rts

;---------------------------------------------------------------------
; List_Previous(BYTE *structure)
;
; Function: Get a pointer to the structure in <list> that points to
; <structure>
;
; Output: a0.l = *previous structure
;---------------------------------------------------------------------
List_Previous:
prosit@0011:
	movem.l		a1,-(sp)
	move.l		4+1*4(sp),a0	;*structure
	move.l		a0,a1		;Store *structure
\loop	cmp.l		LIST_NEXT(a0),a1
	beq.s		\done
	move.l		LIST_NEXT(a0),a0
	bra.s		\loop

\done	movem.l		(sp)+,a1
	rts

;---------------------------------------------------------------------
; List_Find(BYTE *list, WORD ID)
;
; Function: Get a pointer to the structure in <list> with <ID>.
;
; Output: a0.l = *structure, 0 if not found
;---------------------------------------------------------------------
List_Find:
prosit@0012:
	movem.l		a1/d0,-(sp)
	move.l		4+2*4(sp),a0
	move.w		4+2*4+4(sp),d0
	move.l		(a0),a0
	move.l		a0,a1
\loop	cmp.w		LIST_ID(a0),d0
	beq.s		\done
	move.l		LIST_NEXT(a0),a0
	cmp.l		a0,a1
	bne.s		\loop
	move.l		#0,a0
\done	movem.l		(sp)+,a1/d0
	rts

;---------------------------------------------------------------------
; List_Size(BYTE *list)
;
; Function: Get number of elements in list.
;
; Output: d0.w = No. of elements in list.
;---------------------------------------------------------------------
List_Size:
prosit@0013:
	movem.l		a0-a1,-(sp)
	move.l		4+2*4(sp),a0	;*List
	moveq		#0,d0
	move.l		(a0),a0
	cmp.l		#0,a0
	beq.s		\done		;List is empty

	move.l		a0,a1
\loop	addq.w		#1,d0
	move.l		(a0),a0
	cmp.l		a0,a1
	bne.s		\loop

\done	movem.l		(sp)+,a0-a1
	rts
