KEYBOARD_READ_DELAY	EQU	4

TIOS_INT5_VALUE		EQU	$B2
PROSIT_INT5_VALUE	EQU	$F4

POINTER_HEIGHT		EQU	8
POINTER_WIDTH		EQU	6
POINTER_X_HOTSPOT	EQU	1
POINTER_Y_HOTSPOT	EQU	1

DummyInt:
	rte


;************************
;* Toggle the scheduler *
;************************
ToggleProsit:
	eor.w		#%0010011100000000,(sp)
	rte

;*****************************************************************
;* This is where we will end up if a program does that final RTS *
;*****************************************************************
ReturnFromProcess:
	move.w		CurrentProcessID,-(sp)
	bsr		Process_Terminate

;****************************************************
;* The main piece of code in the OS - the scheduler *
;****************************************************
Scheduler:
	or.w		#$0700,sr		;De-activate KeyReader

;Save context
SMCurrentProcess:
	movem.l		d0-d7/a0-a6,$12345678	; Save D0-D7 and A0-A6 (self-modifying code)
	move.l		SMCurrentProcess+4,a0

	move.l		usp,a1
	move.l		a1,C_USP-C_D0(a0)	; Save USP
	move.w		(sp)+,C_SR-C_D0(a0)	; Save SR
	move.l		(sp)+,C_PC-C_D0(a0)	; Save PC

	movem.l		(TIOS_CONSTANTS),a1/d0-d7
	movem.l		a1/d0-d7,C_TIOS_CONSTANTS-C_D0(a0)

;Restore context
SelectNextProcess:
	move.l		C_NEXT-C_D0(a0),a0

SelectedNextProcess:
	add.w		#C_D0,a0
	move.w		C_ID-C_D0(a0),CurrentProcessID	; Store process' ID for later use
	move.l		a0,SMCurrentProcess+4	;Store this context's adress using self-modifying code

	movem.l		C_TIOS_CONSTANTS-C_D0(a0),a1/d0-d7
	movem.l		a1/d0-d7,(TIOS_CONSTANTS)

	move.l		C_USP-C_D0(a0),a1
	move.l		a1,usp			; Restore USP
	move.l		C_PC-C_D0(a0),-(sp)	; Restore PC
	move.w		C_SR-C_D0(a0),-(sp)	; Restore SR

	movem.l		C_D0-C_D0(a0),d0-d7/a0-a6	; Restore D0-D7 and A0-A6

	rte					; Activate next process



;*************************
;* The keyreader routine *
;*************************
KeyReader:
	or.w		#$0700,sr		;De-activate Scheduler

	add.l		#1,Ticks

	movem.l		a0/d0-d1/d4/d6-d7,-(sp)

; Control Keys code
	move.w		#%0000001111111110,($600018)
	move.l		(sp),(sp)		;Wait for I/O recovery
	move.l		(sp),(sp)		;Wait for I/O recovery
	move.b		($60001B),d7		;Read keyboard matrix row mask
	move.b		d7,ControlKeys

; "Normal" keyboard code
	sub.w		#1,KeyboardDelay
	bge		\done
	move.w		#KEYBOARD_READ_DELAY,KeyboardDelay

	move.w		#%0000000000000001,($600018)
	move.l		(sp),(sp)		;Wait for I/O recovery
	move.l		(sp),(sp)		;Wait for I/O recovery
	move.b		($60001B),d4		;Read keyboard matrix row mask
	not.b		d4
	bne.s		\cont1
	clr.w		CurrentKey
	bra.s		\done

\cont1	move.l		#$FFFFFF01,d0		;Initial row mask
	moveq		#1,d6			;Initial row index
	moveq		#8,d1			;Shift counter (8-4-2-1)
\loop	rol.l		d1,d0			;Rotate in case of key
	add.w		d1,d6			;Subtract row index in case of key
	move.w		d0,($600018)		;Set keyboard matrix row mask
	move.l		(sp),(sp)		;Wait for I/O recovery
	move.l		(sp),(sp)		;Wait for I/O recovery
	move.b		($60001B),d7		;Read keyboard matrix column mask
	not.b		d7			;Invert to be able to check for "no key"
	bne.s		\cont2			;There are keys, just keep going
	ror.l		d1,d0			;There are no keys, rotate back...
	sub.w		d1,d6			;...and add back row index
\cont2	lsr.w		#1,d1			;Halve the next step
	bne.s		\loop			;Repeat until the step = 0

	move.w		d0,($600018)
	move.l		(sp),(sp)		;Wait for I/O recovery
	move.l		(sp),(sp)		;Wait for I/O recovery
	move.b		($60001B),d1		;Read column mask

	moveq		#7,d7			;Initial column index
\bloop	btst		d7,d1
	beq.s		\found
	dbra.s		d7,\bloop

\found	lsl.w		#3,d6			;Row index = row index * 8 (one column)
	add.w		d7,d6			;Add column index value
	lsl.w		#1,d6			;Because the key code table contains words...
	lea		KeyCodes(pc),a0
	move.w		0(a0,d6.w),CurrentKey	;Get key code

\done	movem.l		(sp)+,a0/d0-d1/d4/d6-d7
	rte





	EVEN

 ifeq Calculator-TI92Plus
MouseDirectionData:
	dc.w	0,0	:Down+Right+Up+Left
	dc.w	0,0	;Down+Right+Up
	dc.w	0,0	;Down+Right+Left
	dc.w	1,1	;Down+Right
	dc.w	0,0	;Down+Right+Left
	dc.w	0,0	;Down+Up
	dc.w	-1,1	;Down+Left
	dc.w	0,1	;Down
	dc.w	0,0	;Right+Up+Left
	dc.w	1,-1	;Right+Up
	dc.w	0,0	;Right+Left
	dc.w	1,0	;Right
	dc.w	-1,-1	;Up+Left
	dc.w	0,-1	;Up
	dc.w	-1,0	;Left
	dc.w	0,0	;No key
KeyCodes:
	dc.w	-1,-2,-3,-4,337,338,340,344	;Down, Right, Up, Left, Hand, Shift, Diamond, 2nd
	dc.w	0,122,115,119,275,49,50,51	;N/A, z, s, w, F8, 1, 2, 3
	dc.w	0,120,100,101,270,52,53,54	;N/A, x, d, e, F3, 4, 5, 6
	dc.w	258,99,102,114,274,55,56,57	;Store, c, f, r, F7, 7, 8, 9
	dc.w	32,118,103,116,269,40,41,44	;Space, v, g, t, F2, (, ), ","
	dc.w	47,98,104,121,273,259,260,261	;"/", b, h, y, F6, Sin, Cos, Tan
	dc.w	94,110,106,117,268,262,-13,112	;"^", n, j, u, F1, ln, Enter2, P
	dc.w	61,109,107,105,272,263,265,42	;"=", m, k, i, F5, Clear, Apps, "*"
	dc.w	257,136,108,111,43,266,264,0	;Backspace, theta, l, o, "+", Mode, ESC, N/A
	dc.w	45,13,97,113,271,48,46,173	;"-", Enter1, a, q, F4, 0, ".", "(-)"
 endif

 ifeq Calculator-TI89
MouseDirectionData:
	dc.w	0,0	:Right+Down+Left+Up
	dc.w	0,0	;Right+Down+Left
	dc.w	0,0	;Right+Down+Up
	dc.w	1,1	;Right+Down
	dc.w	0,0	;Right+Left+Up
	dc.w	0,0	;Right+Left
	dc.w	1,-1	;Right+Up
	dc.w	1,0	;Right
	dc.w	0,0	;Down+Left+Up
	dc.w	-1,1	;Down+Left
	dc.w	0,0	;Down+Up
	dc.w	0,1	;Down
	dc.w	-1,-1	;Left+Up
	dc.w	-1,0	;Left
	dc.w	0,-1	;Up
	dc.w	0,0	;No key
KeyCodes:
	dc.w	-4,-3,-1,-2,344,338,340,-5	;Up, Left, Down, Right, 2nd, Shitf, Diamond, Alpha
	dc.w	13,43,45,42,47,94,263,272	;Enter, "+", "-", "*", "/", "^", Clear, F5
	dc.w	173,51,54,57,44,116,257,271	;"(-)", 3, 6, 9, ",", t, Backspace, F4
	dc.w	46,50,53,56,41,122,278,270	;".", 2, 5, 8, ), <, Catalog, F3
	dc.w	48,49,52,55,40,121,266,269	;0, 1, 4, 7, (, y, Mode, F2
	dc.w	265,258,149,124,61,120,277,268	;Apps ,Store, EE, "|" ,"=", x, Home, F1
	dc.w	264,0,0,0,0,0,0,0		;ESC, N/A, N/A, N/A, N/A, N/A, N/A, N/A
	dc.w	0,0,0,0,0,0,0,0			;N/A, N/A, N/A, N/A, N/A, N/A, N/A, N/A
	dc.w	0,0,0,0,0,0,0,0			;N/A, N/A, N/A, N/A, N/A, N/A, N/A, N/A
	dc.w	0,0,0,0,0,0,0,0			;N/A, N/A, N/A, N/A, N/A, N/A, N/A, N/A
 endif
