;****************
;*              *
;* SYSTEM CALLS *
;*              *
;****************

;	PROSIT_LOCK_HEADER	1

;---------------------------------------------------------------------
; Process_New(WORD filehandle)
;
; Function: Create a new process from the file with a specified handle
;
; Output: d0 = Process ID, -1 if failure
;---------------------------------------------------------------------
Process_New:
prosit@0002:
	movem.l		a0-a2/a4-a6/d1-d7,-(sp)

	move.w		4+13*4(sp),d5		;Get handle of file off the USP

	TOGGLE_PROSIT

	bsr		RelocateFile		;Relocate program. a6.l = *file, a5.l = *_main
	move.w		d4,d0			;d4 is the return value from RelocateFile
	blt		Process_New_Fail

	add.w		2(a6),a6		;*_prosit

	move.b		#CS_INIT_NORMAL,d3	;Status flags for normal process

Process_New_From_Process_Fork:

;**********************************
;* Setup the context memory block *
;**********************************
	move.l		#CONTEXT_SIZE,-(sp)
	jsr		doorsos::HeapAllocHigh	;Allocate memory for context block
	addq.l		#4,sp
	move.w		d0,d6			;Store context block handle
	doorsos::DEREF	d0,a4			;*Context block

	move.l		a6,C_PROSIT_HEADER(a4)	;*Prosit header of process

;Setup the user stack
	moveq		#0,d0
	move.w		#PROSIT_STACKSIZE,-(sp)
	move.l		a6,-(sp)
	bsr		Header_FindField	;a0 = *stacksize field
	cmp.l		#0,a0
	beq.s		\nostk			;No stack field, don't allocate
	move.w		2(a0),d0
	beq.s		\nostk			; Stack size = zero, don't allocate
	move.w		d0,d7
	move.l		d0,-(sp)
	jsr		doorsos::HeapAllocHigh	; Allocate stack
	addq.l		#4,sp
\nostk	move.w		d0,C_STACK_HANDLE(a4)	; Set stack handle, zero if no stack
	beq.s		\sdone
	doorsos::DEREF	d0,a2			; *Stack
	lea		0(a2,d7.w),a2		; A stack starts at the top
	move.l		#ReturnFromProcess,-(a2); Set return point if process self-terminates
	move.l		a2,C_USP(a4)		; Set USP
\sdone	move.b		d3,C_STATUS_FLAGS(a4)	; Set process' status flags
	tst.w		d4
	ble.s		\noarc
	bset.b		#CS_COPY_OF_ARCHIVED,C_STATUS_FLAGS(a4)	;Fiel is a copy from archive memory
;Setup the execution parameters
\noarc	move.w		d6,C_ID(a4)		; Set ID
	move.w		#0,C_SR(a4)		; Set SR
	move.l		a5,C_PC(a4)		; Set PC
	move.w		d5,C_PROG_HANDLE(a4)	; Set program handle

;**************************
;* Insert process in list *
;**************************
	move.b		#0,-(sp)		;List item priority
	move.l		a4,-(sp)		;*List item
	move.l		#ContextList,-(sp)	;*List
	bsr		List_Insert
	lea		10(sp),sp

	move.w		d6,d0			;Return value

Process_New_Fail:
	TOGGLE_PROSIT

	movem.l		(sp)+,a0-a2/a4-a6/d1-d7
	rts

;---------------------------------------------------------------------
; Process_Fork(BYTE *header, BYTE *code)
;
; Function: Fork an existing process
;
; Output: d0 = Process ID, -1 if failure
;---------------------------------------------------------------------
Process_Fork:
prosit@0003:
	movem.l		a0-a2/a4-a6/d1-d7,-(sp)
	move.w		CurrentProcessID,d5
	doorsos::DEREF	d5,a5
	move.b		C_STATUS_FLAGS(a5),d3	;Get parent process' flags
	and.b		#CS_FORK_KEEP_FLAGS,d3	;Keep some flags...
	or.b		#CS_INIT_FORK,d3	;...and add some.
	move.w		C_PROG_HANDLE(a5),d5	;Use parent program's file handle

	move.w		d5,d4
	doorsos::DEREF	d4,a5		;Get adress of program variable
	add.w		#1,10(a5)	;Increase parent program's relocation counter

	moveq		#0,d4			;Forks can't be archived
	move.l		4+13*4+4(sp),a5		;*code
	move.l		4+13*4(sp),a6		;*header
	TOGGLE_PROSIT
	bra		Process_New_From_Process_Fork


;---------------------------------------------------------------------
; Process_Terminate(WORD ID)
;
; Function: Terminate process
;
; Output: d0 = 0 if success, -1 if failure
;---------------------------------------------------------------------
Process_Terminate:
prosit@0004:
	movem.l		a0-a4/d1-d2/d6,-(sp)

	move.w		4+8*4(sp),d6	;ID
	TOGGLE_PROSIT

	move.w		d6,-(sp)
	move.l		#ContextList,-(sp)
	bsr		List_Find
	addq.l		#6,sp
	cmp.l		#0,a0
	beq		\done		;Process not found in list

	move.l		LIST_NEXT(a0),a4	;This pointer will be removed in List_Remove!

	move.l		a0,-(sp)
	move.l		#ContextList,-(sp)
	bsr		List_Remove
	addq.l		#8,sp

	move.l		a0,a3

	move.w		C_PROG_HANDLE(a3),d0

;	btst.b		#CS_FORK,C_STATUS_FLAGS(a3)	;Fork?
;	beq.s		\prog				;Nope
;Process was a fork and shouldn't be unrelocated
	move.w		d0,d1
	doorsos::DEREF	d1,a0		;Get adress of program variable
	sub.w		#1,10(a0)	;Decrease program's relocation counter
	bgt.s		\cont

;	bra.s		\cont

;	move.w		d0,d1
;	doorsos::DEREF	d1,a0		;Get adress of program variable
;	sub.w		#1,8+2(a0)	;Decrease parent program's relocation counter
;	bgt.s		\cont		;Still code running inside!

	move.w		#1,10(a0)
\prog	move.l		($40),a0
	jsr		(a0)			;Unrelocate program

;	btst.b		#CS_FORK,C_STATUS_FLAGS(a3)	;Fork?
;	bne.s		\cont				;Yep, don't de-allocate

	btst.b		#CS_COPY_OF_ARCHIVED,C_STATUS_FLAGS(a3)	;Copy of archived file?
	beq.s		\cont					;No, continue
	move.w		C_PROG_HANDLE(a3),-(sp)
	jsr		doorsos::HeapFree			;Yes, de-allocate it
	addq.l		#2,sp

\cont	move.w		C_STACK_HANDLE(a3),-(sp)
	beq.s		\nostk
	jsr		doorsos::HeapFree	;De-allocate stack memory block

\nostk	move.w		d6,(sp)			;Not -(sp) because we've already used the stack!
	jsr		doorsos::HeapFree	;De-allocate context memory block
	addq.l		#2,sp

;\l	bra.s	\l

	move.l		a4,a0
	cmp.w		CurrentProcessID,d6	;Self-terminated process?
	beq		SelectedNextProcess	;...yes, move on to the next one

	moveq		#0,d0			;Success

\done	TOGGLE_PROSIT

	movem.l		(sp)+,a0-a4/d1-d2/d6
	rts

;---------------------------------------------------------------------
; Sleep(void)
;
; Function: Put process to sleep, i.e. it's moved from the list of
;  executed processes to the list of sleeping processes. It will
;  automatically wake up when a relevant event is triggered.
;---------------------------------------------------------------------
Sleep:
prosit@0005:
	TOGGLE_PROSIT

	move.w		sr,-(sp)		;Must be stored since the code below might change SR...
	or.w		#$0700,sr		;De-activate KeyReader

	move.l		a0,-(sp)
	move.l		SMCurrentProcess+4,a0
	movem.l		d0-d7/a0-a6,C_D0-C_D0(a0)	; Save D0-D7 and A0-A6
	move.l		(sp)+,C_A0-C_D0(a0)		;Restore process' a0

	move.w		(sp)+,d0		;Old SR with supervisor mode and no interrupts
	and.w		#$D8FF,d0		;User mode, turn on interrupts
	move.w		d0,C_SR-C_D0(a0)

	move.l		usp,a1
	move.l		(a1)+,C_PC-C_D0(a0)	;PC = RTS adress
	move.l		a1,C_USP-C_D0(a0)	;...and store the "new" USP.
	bset.b		#CS_SLEEPING,C_STATUS_FLAGS-C_D0(a0)

;	sub.l		#C_D0,a0
;	HexShow		C_SR-2(a0),#2,#16,#3
;	add.l		#C_D0,a0

	move.b		#0,-(sp)
	pea		-C_D0(a0)
	move.l		C_NEXT-C_D0(a0),a0	;Where the scheduler will pick things up
	move.l		#ContextList,-(sp)
	bsr		List_Remove		;Remove process from "running" list...
	move.l		#SleepingContextList,(sp)
	bsr		List_Insert		;...and put it in the "sleeping" list.
	lea		10(sp),sp

	bra		SelectedNextProcess

;\back	movem.l		(sp)+,a0-a1
;	rts

;---------------------------------------------------------------------
; Wake_Up(WORD ID)
;
; Function: Wake up process <ID>, i.e. move it from the list of
;  sleeping processes to the list of executed processes. If the
;  process isn't sleeping nothing happens.
;---------------------------------------------------------------------
Wake_Up:
prosit@0006:
	movem.l		a0/d0,-(sp)
	move.w		4+2*4(sp),d0		;ID
	doorsos::DEREF	d0,a0
	btst.b		#CS_SLEEPING,C_STATUS_FLAGS(a0)
	beq.s		\done			;Process isn't sleeping!

	TOGGLE_PROSIT
	or.w		#$0700,sr		;De-activate KeyReader

	move.b		#0,-(sp)
	move.l		a0,-(sp)
	move.l		#SleepingContextList,-(sp)
	bsr		List_Remove
	move.l		#ContextList,(sp)
	bsr		List_Insert
	lea		10(sp),sp

	bclr.b		#CS_SLEEPING,C_STATUS_FLAGS(a0)

;	HexShow		a0,#3,#7,#7
;	HexShow		C_SR-2(a0),#3,#16,#3

	TOGGLE_PROSIT

\done	movem.l		(sp)+,a0/d0
	rts

;---------------------------------------------------------------------
; Trigger_Event(WORD processID, LONG event)
;
; Function: Execute the <event> subroutine inside <processID>.
;
; Output: d0.w >= 0 if success, -1 if failed
;---------------------------------------------------------------------

Trigger_Event:
prosit@0007:
	movem.l		a0-a3/d5-d7,-(sp)

	move.w		4+7*4(sp),d6		;processID

	move.w		d6,d5
	doorsos::DEREF	d5,a1			;*context
	move.l		4+7*4+2(sp),d5		;event

	move.w		#PROSIT_EVENT,-(sp)
	move.l		C_PROSIT_HEADER(a1),-(sp)
\loop	bsr		Header_FindField	;a0 = *First event
	cmp.l		#0,a0
	beq		\nfound			;Event not found, we're done
	cmp.l		2(a0),d5
	beq.s		\found			;Event found!
	move.l		a0,(sp)
	bra.s		\loop

\nfound	addq.l		#6,sp
	bra		\done

\found	addq.l		#6,sp

;Wait until currently running event is done
\floop	btst.b		#CS_IN_INTERRUPT,C_STATUS_FLAGS(a1)
	beq.s		\ok			;No event is currently running, go on
	BREAK_AND_COOPERATE			;Event is running, wait for it to finish
	bra.s		\floop

;Wait for process to return from an eventual library call
\ok	move.w		C_PROG_HANDLE(a1),d7
	doorsos::DEREF	d7,a2		;Start of program file
	move.l		a2,a3
	add.w		(a2),a3		;End of program file
\stick	cmp.l		C_PC(a1),a2
	bgt.s		\wait
	cmp.l		C_PC(a1),a3
	bgt.s		\trig
\wait	BREAK_AND_COOPERATE
	bra.s		\stick

\trig	TOGGLE_PROSIT
	bset.b		#CS_IN_INTERRUPT,C_STATUS_FLAGS(a1)
	move.l		C_USP(a1),a2			;Get process' current SP
	move.l		C_PC(a1),-(a2)			;Push process' PC onto its stack
	move.w		C_SR(a1),-(a2)			;Push process' SR onto its stack
	move.l		#Return_From_Event,-(a2)	;Where we will return after an event
	move.l		a2,C_USP(a1)			;Store process' new SP
	move.l		6(a0),C_PC(a1)			;Point process' PC to the event handler
	TOGGLE_PROSIT

	moveq		#0,d0
	btst.b		#CS_SLEEPING,C_STATUS_FLAGS(a1)
	beq.s		\done				;Process isn't sleeping
;Process is sleeping, wake it up
	move.w		d6,-(sp)
	bsr		Wake_Up
	addq.l		#2,sp

\done	movem.l		(sp)+,a0-a3/d5-d7
	rts

Return_From_Event:
	TOGGLE_PROSIT

	subq.l		#6,sp			;Make room on the stack
	movem.l		a1,-(sp)		;Store registers that are used here

	move.l		SMCurrentProcess+4,a1
	bclr.b		#CS_IN_INTERRUPT,C_STATUS_FLAGS-C_D0(a1)

	move.l		usp,a1			;Process' SP
	move.w		(a1)+,4(sp)		;Process' SR
	move.l		(a1)+,4+2(sp)		;Process' PC
	move.l		a1,usp			;Restore process' USP

	movem.l		(sp)+,a1		;Restore used registers

	rte				;Return to process

;---------------------------------------------------------------------
; Broadcast_Event(LONG event)
;
; Function: Execute the <event> subroutine inside all processes
;  except the caller.
;---------------------------------------------------------------------
Broadcast_Event:
prosit@0008:
	movem.l		a0-a2/d0-d2,-(sp)

	move.l		4+6*4(sp),d2		;event

;Running processes
	move.l		ContextList,a0		;*First
	cmp.l		#0,a0
	beq.s		\sleep

\rloop	move.w		C_ID(a0),d0		;ProcessID
	move.l		C_NEXT(a0),a0
	cmp.w		CurrentProcessID,d0
	beq		\rnext			;The caller must not get the event!
	move.l		d2,-(sp)
	move.w		d0,-(sp)
	bsr		Trigger_Event
	addq.l		#6,sp
\rnext	cmp.l		ContextList,a0
	bne.s		\rloop			;Next process


;Sleeping processes
\sleep	move.l		#SleepingContextList,-(sp)
	bsr		List_Size
	addq.l		#4,sp
	subq.w		#1,d0
	blt.s		\done			;List is empty

	move.l		SleepingContextList,a0		;*First
	move.w		d0,d1
\sloop	move.w		C_ID(a0),d0		;ProcessID
	move.l		C_NEXT(a0),a0		;Get thsi first in case the proecss is waken up
	cmp.w		CurrentProcessID,d0
	beq		\snext			;The caller must not get the event!
	move.l		d2,-(sp)
	move.w		d0,-(sp)
	bsr		Trigger_Event
	addq.l		#6,sp
\snext	dbra.s		d1,\sloop		;Next process
;	cmp.l		SleepingContextList,a0
;	bne.s		\sloop			;Next process

\done	movem.l		(sp)+,a0-a2/d0-d2
	rts
