;
; IDEA: Forced termination should jump to the _exit point of the program that is to be terminated!
;
; IDEA: A process is only allowed to send one message per execution slot. Can easily be done by
;        jumping to the scheduler after the message has been inserted and the relevant registers
;        have been restored (including return values).
;
; DON'T FORGET to add a6.l as an input to Process_Fork (PROSIT_RETURNDATA!)
;

	include		"calc.h"

	xdef    	_main

	xdef		_exit
	xdef		_comment

	include		"doorsos.h"
	include		"macros.h"
	include		"hexlib.h"

	xdef		prosit@0000	;Header_Get
	xdef		prosit@0001	;Header_FindField

	xdef		prosit@0002	;Process_New
	xdef		prosit@0003	;Process_Fork
	xdef		prosit@0004	;Process_Terminate

	xdef		prosit@0005	;Sleep
	xdef		prosit@0006	;Wake_Up
	xdef		prosit@0007	;Trigger_Event
	xdef		prosit@0008	;Broadcast_Event

	xdef		prosit@0009	;Get_File_Handle (p_file.asm)
	xdef		prosit@000A	;Execute_File (p_file.asm)

	xdef		prosit@000B	;CurrentProcessID
	xdef		prosit@000C	;Ticks
	xdef		prosit@000D	;ControlKeys
	xdef		prosit@000E	;CurrentKey

	xdef		prosit@000F	;List_Insert
	xdef		prosit@0010	;List_Remove
	xdef		prosit@0011	;List_Previous
	xdef		prosit@0012	;List_Find
	xdef		prosit@0013	;List_Size

	include		"prosit.h"	;Import system call equ:s

INT1_ADDRESS		EQU	$64
INT4_ADDRESS		EQU	$70
INT5_ADDRESS		EQU	$74
TRAP13_ADDRESS		EQU	$B4
TRAP14_ADDRESS		EQU	$B8

C_NEXT			EQU	0
C_ID			EQU	C_NEXT+4
C_PRIORITY		EQU	C_ID+2
C_STATUS_FLAGS		EQU	C_PRIORITY+1
C_D0			EQU	C_STATUS_FLAGS+1		; D0
C_A0			EQU	C_D0+32			; A0
C_USP			EQU	C_A0+28			; USP
C_SR			EQU	C_USP+4			; SR
C_PC			EQU	C_SR+2			; PC

;Process starts of as "running"
CS_SLEEPING		EQU	0
CS_IN_INTERRUPT		EQU	1
CS_COPY_OF_ARCHIVED	EQU	2
CS_FORK			EQU	3
CS_INIT_NORMAL		EQU	%00000000
CS_INIT_FORK		EQU	%00001000
CS_FORK_KEEP_FLAGS	EQU	%00000100
	;Bit 0: Sleeping
	;Bit 1: In interrupt
	;Bit 2: Copy of archived program
	;Bit 3: Fork
	;Bit 4:
	;Bit 5:
	;Bit 6: Library Call lock
	;Bit 7: Event lock

;IF THESE FLAGS ARE CHANGED: CHECK ALL THE Process_* functions!


C_PROG_HANDLE		EQU	C_PC+4			; Program file handle
C_STACK_HANDLE		EQU	C_PROG_HANDLE+2		; Stack handle
C_PROSIT_HEADER		EQU	C_STACK_HANDLE+2	; *Prosit header
C_TIOS_CONSTANTS	EQU	C_PROSIT_HEADER+4
CONTEXT_SIZE		EQU	C_TIOS_CONSTANTS+9*4


_main:

;Find and relocate ProBoot process
	pea		folder(pc)		;*Folder name
	pea		program(pc)		;*File name
	bsr		Get_File_Handle		;d5.w = File handle
	addq.l		#8,sp
	move.w		d5,ProBootFileHandle
	beq		_exit			;File didn't exist, quit

	bsr		RelocateFile		;Relocate file - a5=*_main, a6=*file start

	add.w		2(a6),a6		;*_prosit

;Got all necessary information, create context entry
	move.l		#CONTEXT_SIZE,-(sp)
	jsr		doorsos::HeapAllocHigh
	addq.l		#4,sp

	move.w		d0,CurrentProcessID	;Store handle for later use :)
	doorsos::DEREF	d0,a0			;Get adress of handle

	move.w		CurrentProcessID,C_ID(a0)
	move.w		ProBootFileHandle,C_PROG_HANDLE(a0)
	clr.w		C_STACK_HANDLE(a0)
	move.l		a6,C_PROSIT_HEADER(a0)
	move.b		#CS_INIT_NORMAL,C_STATUS_FLAGS(a0)

;Setup context table
	clr.l		ContextList
	clr.l		SleepingContextList
	move.b		#0,-(sp)
	move.l		a0,-(sp)
	move.l		#ContextList,-(sp)
	bsr		List_Insert
	lea		10(sp),sp

	add.w		#C_D0,a0
	move.l		a0,SMCurrentProcess+4	;Self-modifying code for the scheduler

;Set some variables to their default values
	clr.w		CurrentKey
	move.b		#$FF,ControlKeys
	clr.w		KeyboardDelay
	clr.l		Ticks

;***************************************************************
;* Initialize the scheduler:                                   *
;*  o Point Auto-Int 1 to the Scheduler                        *
;*  o Set Auto-Int 5:s frequency and point it to the KeyReader *
;*  o Point Trap #13 to ToggleProsit                           *
;*  o Point Trap #14 to the Scheduler                          *
;***************************************************************
	move.w		#$0700,d0
	trap		#1
	move.l		INT1_ADDRESS,old_int1
	move.l		INT4_ADDRESS,old_int4
	move.l		INT5_ADDRESS,old_int5
	move.l		TRAP13_ADDRESS,old_trap13
	move.l		TRAP14_ADDRESS,old_trap14
	bclr.b		#2,$600001
	move.l		#Scheduler,INT1_ADDRESS
	move.l		#DummyInt,INT4_ADDRESS
	move.l		#KeyReader,INT5_ADDRESS
	move.l		#ToggleProsit,TRAP13_ADDRESS
	move.l		#Scheduler,TRAP14_ADDRESS
	bset.b		#2,$600001
	trap		#1
	move.w		#PROSIT_INT5_VALUE,($600016)

;***********************
;* Run ProBoot process *
;***********************
	jsr		(a5)
\wait	;tst.l		ContextList
	;bne.s		\wait			;Wait for all processes to terminate
	tst.w		CurrentKey
	bne.s		\wait			;Wait for the user to release an eventual key

;****************************
;* Deactivate the scheduler *
;****************************



;REDIRECT TRAP #1 ???



	move.w		#$0700,d0
	trap		#1
	bclr.b		#2,$600001
	move.l		old_int1,INT1_ADDRESS
	move.l		old_int4,INT4_ADDRESS
	move.l		old_int5,INT5_ADDRESS
	move.l		old_trap13,TRAP13_ADDRESS
	move.l		old_trap14,TRAP14_ADDRESS
	bset.b		#2,$600001
	trap		#1
	move.w		#TIOS_INT5_VALUE,($600016)

	move.w		ProBootFileHandle,d0
	move.l		($40),a0
	jsr		(a0)			;Unrelocate ProBoot file

;******************************************************
;* De-allocate ProBoot context block and context list *
;******************************************************
	move.w		CurrentProcessID,-(sp)
	jsr		doorsos::HeapFree	;De-allocate ProBoot context block
	addq.l		#2,sp

	jsr		doorsos::PortRestore	;Restore the LCD port region for TIOS

_exit:
	rts





	include		"p_header.asm"
	include		"p_syscal.asm"
	include		"p_sched.asm"
	include		"p_file.asm"
	include		"p_list.asm"

;***********************************************************************


;***********************************************************************

;***********************************************************************

	EVEN
prosit@000B:
CurrentProcessID	dc.w	0
prosit@000C:
Ticks			dc.l	0
prosit@000D:
ControlKeys		dc.w	0
prosit@000E:
CurrentKey		dc.w	0


folder		dc.b	"main",0
program		dc.b	"proboot",0

_comment
	dc.b	"Prosit OS launcher",0

	BSS

ProBootFileHandle	dc.w	0
ContextList		dc.l	0
SleepingContextList	dc.l	0

old_int1		dc.l	0
old_int4		dc.l	0
old_int5		dc.l	0
old_trap13		dc.l	0
old_trap14		dc.l	0

KeyboardDelay		dc.w	0

	END
